<?php

namespace Drupal\commerce_affirm\EventSubscriber;

use Drupal\commerce_affirm\Plugin\Commerce\PaymentGateway\Redirect;
use Drupal\commerce_payment\Entity\PaymentGatewayInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Subscribes to order events.
 */
class OrderSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * Constructs a new OrderSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      'commerce_order.place.post_transition' => ['onOrderPlaced', -20],
    ];
  }

  /**
   * Updates payment after placing an order.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   *   The event.
   */
  public function onOrderPlaced(WorkflowTransitionEvent $event): void {
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $event->getEntity();

    $payment_gateway = $order->get('payment_gateway');
    if ($payment_gateway->isEmpty() || !$payment_gateway->entity instanceof PaymentGatewayInterface) {
      return;
    }

    $payment_gateway_plugin = $payment_gateway->entity->getPlugin();
    if (!$payment_gateway_plugin instanceof Redirect) {
      return;
    }

    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    $payments = $payment_storage->loadMultipleByOrder($order);
    if (empty($payments)) {
      return;
    }

    $payment = reset($payments);
    $payment_gateway_plugin->updatePayment($order, $payment);
  }

}
